@@ -1,26 +1,27 @@
 """Localization."""
 from __future__ import unicode_literals
 import gettext
-from ... import util
+import os
 
 lang = None
 current_domain = None
+locale_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
 
 
 def _(text):
-    """Unicode gettext."""
+    """Unicode `gettext`."""
 
-    return util.translate(lang, text) if lang is not None else text
+    return lang.gettext(text) if lang is not None else text
 
 
-def setup(domain, pth, language=None):
+def setup(domain, language=None):
     """Setup a language."""
 
     global lang
     global current_domain
     if language is not None:
         try:
-            lang = gettext.translation(domain, pth, languages=[language])
+            lang = gettext.translation(domain, locale_path, languages=[language])
             lang.install()
             current_domain = domain
         except Exception:
@@ -44,5 +45,5 @@ def get_current_domain():
     return current_domain
 
 
-# Init the default setup on intitial load
+# Initialize the default setup on initial load
 _default_setup()
