@@ -26,7 +26,8 @@ import os
 import functools
 import codecs
 import time
-from backrefs import bre, bregex
+from backrefs import bre
+from collections import deque
 from . import gui
 from .controls.pick_button import PickButton, pick_extend
 from .settings import Settings
@@ -34,17 +35,38 @@ from . import data
 from .localization import _
 from .. import rumcore
 from .. import util
+if rumcore.REGEX_SUPPORT:
+    from backrefs import bregex
+else:
+    bregex = None
+
+
+class PluginException(Exception):
+    """Plugin exception."""
 
 
 class RegexTestDialog(gui.RegexTestDialog):
     """Regex test dialog."""
 
     def __init__(self, parent):
-        """Init Regex Test Dialog object."""
+        """Initialize Regex Test Dialog object."""
 
         super(RegexTestDialog, self).__init__(None)
         if util.platform() == "windows":
             self.SetDoubleBuffered(True)
+
+        test_attr = self.m_test_text.GetDefaultStyle()
+        self.highlight_attr = test_attr.Merge(
+            test_attr,
+            wx.TextAttr(wx.NullColour, colBack=wx.Colour(0xFF, 0xCC, 0x00))
+        )
+        self.test_attr = test_attr.Merge(
+            test_attr,
+            wx.TextAttr(wx.NullColour, colBack=wx.Colour(0xFF, 0xFF, 0xFF))
+        )
+        self.replace_attr = self.m_replace_text.GetDefaultStyle()
+        self.error_attr = self.replace_attr.Merge(self.replace_attr, wx.TextAttr(wx.Colour(0xFF, 0, 0)))
+
         self.localize()
 
         self.SetIcon(
@@ -53,9 +75,9 @@ class RegexTestDialog(gui.RegexTestDialog):
         self.parent = parent
         self.regex_mode = Settings.get_regex_mode()
         self.regex_version = Settings.get_regex_version()
-        self.imported_plugin = {}
+        self.imported_plugin = None
 
-        # Ensure OS selectall shortcut works in text inputs
+        # Ensure OS select all shortcut works in text inputs
         self.set_keybindings(
             [
                 (wx.ACCEL_CMD if util.platform() == "osx" else wx.ACCEL_CTRL, ord('A'), self.on_textctrl_selectall)
@@ -99,6 +121,7 @@ class RegexTestDialog(gui.RegexTestDialog):
 
         if not self.parent.m_replace_plugin_checkbox.GetValue():
             self.m_replace_plugin_dir_picker.Hide()
+            self.m_reload_button.Hide()
 
         self.regex_event_code = -1
         self.testing = False
@@ -135,7 +158,7 @@ class RegexTestDialog(gui.RegexTestDialog):
         self.BEST_MATCH = _("Best fuzzy match")
         self.FUZZY_FIT = _("Improve fuzzy fit")
         self.WORD = _("Unicode word break")
-        self.REVERSE = _("Reverse match")
+        self.REVERSE = _("Search backwards")
         self.POSIX = _("Use POSIX matching")
         self.FORMAT = _("Format style replacements")
         self.FULLCASE = _("Full case-folding")
@@ -146,6 +169,7 @@ class RegexTestDialog(gui.RegexTestDialog):
         self.USE_REPLACE_PLUGIN = _("Use replace plugin")
         self.REPLACE_PLUGIN = _("Replace plugin")
         self.REPLACE = _("Replace")
+        self.RELOAD = _("Reload")
 
     def refresh_localization(self):
         """Localize dialog."""
@@ -174,10 +198,11 @@ class RegexTestDialog(gui.RegexTestDialog):
             self.m_replace_label.SetLabel(self.REPLACE_PLUGIN)
         else:
             self.m_replace_label.SetLabel(self.REPLACE)
+        self.m_reload_button.SetLabel(self.RELOAD)
         self.Fit()
 
     def init_regex_timer(self):
-        """Init the update Timer object]."""
+        """Initialize the update Timer object]."""
 
         self.regex_timer = wx.Timer(self)
         self.Bind(wx.EVT_TIMER, self.regex_event, self.regex_timer)
@@ -209,7 +234,7 @@ class RegexTestDialog(gui.RegexTestDialog):
             event.Skip()
 
     def set_keybindings(self, keybindings):
-        """Set keybindings for frame."""
+        """Set key bindings for frame."""
 
         tbl = []
         for binding in keybindings:
@@ -226,8 +251,8 @@ class RegexTestDialog(gui.RegexTestDialog):
         # Reset Colors
         self.m_test_text.SetStyle(
             0,
-            self.m_test_text.GetLastPosition(),
-            wx.TextAttr(wx.Colour(0, 0, 0), colBack=wx.Colour(255, 255, 255))
+            util.ulen(self.m_test_text.GetValue()),
+            self.test_attr
         )
 
     def import_plugin(self, script):
@@ -235,26 +260,30 @@ class RegexTestDialog(gui.RegexTestDialog):
 
         import imp
 
-        if script not in self.imported_plugin:
-            self.imported_plugin = {}
-            module = imp.new_module(script)
-            with open(script, 'rb') as f:
-                encoding = rumcore.text_decode._special_encode_check(f.read(256), '.py')
-            with codecs.open(script, 'r', encoding=encoding.encode) as f:
-                exec(
-                    compile(
-                        f.read(),
-                        script,
-                        'exec'
-                    ),
-                    module.__dict__
-                )
+        try:
+            if self.imported_plugin is None or script != self.imported_plugin[0]:
+                self.imported_plugin = None
+                module = imp.new_module(os.path.splitext(os.path.basename(script))[0])
+                module.__dict__['__file__'] = script
+                with open(script, 'rb') as f:
+                    encoding = rumcore.text_decode._special_encode_check(f.read(256), '.py')
+                with codecs.open(script, 'r', encoding=encoding.encode) as f:
+                    exec(
+                        compile(
+                            f.read(),
+                            script,
+                            'exec'
+                        ),
+                        module.__dict__
+                    )
 
-            # Don't let the module get garbage collected
-            # We will remove references when we are done with it.
-            self.imported_plugin[script] = module
+                # Don't let the module get garbage collected
+                # We will remove references when we are done with it.
+                self.imported_plugin = (script, module)
 
-        return self.imported_plugin[script].get_replace()
+            return self.imported_plugin[1].get_replace()
+        except Exception:
+            raise PluginException(str(traceback.format_exc()))
 
     def test_regex(self):
         """Test and highlight search results in content buffer."""
@@ -279,11 +308,10 @@ class RegexTestDialog(gui.RegexTestDialog):
         if not self.testing:
             self.testing = True
             if self.m_regex_text.GetValue() == "":
-                self.m_test_text.SetStyle(
-                    0,
-                    self.m_test_text.GetLastPosition(),
-                    wx.TextAttr(wx.Colour(0, 0, 0), colBack=wx.Colour(255, 255, 255))
-                )
+                self.reset_highlights()
+                self.m_test_replace_text.Clear()
+                self.m_test_replace_text.SetDefaultStyle(self.replace_attr)
+                self.m_test_replace_text.SetValue(self.m_test_text.GetValue())
                 self.testing = False
                 return
 
@@ -347,8 +375,7 @@ class RegexTestDialog(gui.RegexTestDialog):
                     flags |= engine.UNICODE
                     rum_flags |= rumcore.UNICODE
                 else:
-                    if util.PY3:
-                        flags |= engine.ASCII
+                    flags |= engine.ASCII
                     rum_flags |= rumcore.ASCII
                 if is_regex:
                     if self.m_dotmatch_checkbox.GetValue():
@@ -360,19 +387,20 @@ class RegexTestDialog(gui.RegexTestDialog):
 
             try:
                 if self.regex_mode == rumcore.BREGEX_MODE:
-                    test = bregex.compile_search(search_text, flags)
+                    test = bregex.compile(search_text, flags)
                 elif self.regex_mode == rumcore.REGEX_MODE:
                     test = regex.compile(search_text, flags)
                 elif self.regex_mode == rumcore.BRE_MODE:
-                    test = bre.compile_search(search_text, flags)
+                    test = bre.compile(search_text, flags)
                 else:
                     test = re.compile(search_text, flags)
-            except Exception:
+            except Exception as e:
                 self.reset_highlights()
-                self.m_test_replace_text.SetValue(
-                    self.m_test_text.GetValue() if self.m_replace_text.GetValue() else ''
-                )
                 self.testing = False
+                text = str(e)
+                self.m_test_replace_text.Clear()
+                self.m_test_replace_text.SetDefaultStyle(self.error_attr)
+                self.m_test_replace_text.SetValue(text)
                 return
 
             text = self.m_test_text.GetValue()
@@ -386,18 +414,18 @@ class RegexTestDialog(gui.RegexTestDialog):
                     file_info = rumcore.FileInfoRecord(
                         0,
                         "TestBuffer.txt",
+                        "txt",
                         len(text),
                         time.ctime(),
                         time.ctime(),
                         rumcore.text_decode.Encoding('unicode', None)
                     )
-                    replace_test = self.import_plugin(rpattern)(file_info, rum_flags).replace
-                elif rpattern:
+                    replace_test = self.import_plugin(rpattern)(file_info, rum_flags)._test
+                elif not self.m_replace_plugin_checkbox.GetValue():
                     if not is_regex:
                         replace_test = functools.partial(replace_literal, replace=rpattern)
                     elif self.regex_mode == rumcore.BREGEX_MODE:
-                        replace_test = bregex.compile_replace(
-                            test,
+                        replace_test = test.compile(
                             rpattern,
                             bregex.FORMAT if self.m_format_replace_checkbox.GetValue() else 0
                         )
@@ -408,41 +436,87 @@ class RegexTestDialog(gui.RegexTestDialog):
                         else:
                             replace_test = functools.partial(replace_regex, replace=rpattern)
                     elif self.regex_mode == rumcore.BRE_MODE:
-                        replace_test = bre.compile_replace(
-                            test,
+                        replace_test = test.compile(
                             rpattern,
                             (bre.FORMAT if self.m_format_replace_checkbox.GetValue() else 0)
                         )
                     else:
                         rpattern = util.preprocess_replace(rpattern)
                         replace_test = functools.partial(replace_re, replace=rpattern)
+            except PluginException:
+                self.imported_plugin = None
+                self.testing = False
+                text = str(traceback.format_exc())
+                self.m_test_replace_text.Clear()
+                self.m_test_replace_text.SetDefaultStyle(self.error_attr)
+                self.m_test_replace_text.SetValue(text)
+                return
             except Exception as e:
-                print(e)
-                pass
+                self.imported_plugin = None
+                self.testing = False
+                text = str(e)
+                self.m_test_replace_text.Clear()
+                self.m_test_replace_text.SetDefaultStyle(self.error_attr)
+                self.m_test_replace_text.SetValue(text)
+                return
 
             try:
                 # Reset Colors
                 self.reset_highlights()
 
-                new_text = []
-                offset = 0
-                for m in test.finditer(text):
-                    try:
-                        if replace_test:
-                            new_text.append(text[offset:m.start(0)])
-                            new_text.append(replace_test(m))
-                            offset = m.end(0)
-                    except Exception as e:
-                        print(e)
-                        replace_test = None
-                    self.m_test_text.SetStyle(
-                        m.start(0),
-                        m.end(0),
-                        wx.TextAttr(wx.Colour(0, 0, 0), colBack=wx.Colour(0xFF, 0xCC, 0x00))
-                    )
-                if replace_test:
+                if rumcore.REGEX_SUPPORT and isinstance(test, (bregex._REGEX_TYPE, bregex.Bregex)):
+                    reverse = bool(test.flags & regex.REVERSE)
+                else:
+                    reverse = False
+                new_text = deque()
+                offset = len(text) if reverse else 0
+                byte_start = util.ulen(text) if reverse else 0
+                byte_end = byte_start
+                errors = False
+                try:
+                    for m in test.finditer(text):
+                        try:
+                            if reverse:
+                                value = text[m.end(0):offset]
+                                byte_end = byte_start - util.ulen(value)
+                                byte_start = byte_end - util.ulen(m.group(0))
+                                offset = m.start(0)
+                            else:
+                                value = text[offset:m.start(0)]
+                                byte_start = byte_end + util.ulen(value)
+                                byte_end = byte_start + util.ulen(m.group(0))
+                                offset = m.end(0)
+                            if replace_test:
+                                if reverse:
+                                    new_text.appendleft(value)
+                                    new_text.appendleft(replace_test(m))
+                                else:
+                                    new_text.append(value)
+                                    new_text.append(replace_test(m))
+
+                        except Exception as e:
+                            if not errors:
+                                new_text = [str(e)]
+                                replace_test = None
+                                errors = True
+
+                        self.m_test_text.SetStyle(
+                            byte_start,
+                            byte_end,
+                            self.highlight_attr
+                        )
+                except Exception as e:
+                    new_text = [str(e)]
+                    replace_test = None
+                    errors = True
+                if reverse:
+                    new_text.appendleft(text[:offset])
+                else:
                     new_text.append(text[offset:])
-                self.m_test_replace_text.SetValue(''.join(new_text))
+                value = ''.join(list(new_text))
+                self.m_test_replace_text.Clear()
+                self.m_test_replace_text.SetDefaultStyle(self.error_attr if errors else self.replace_attr)
+                self.m_test_replace_text.SetValue(value)
 
             except Exception:
                 print(str(traceback.format_exc()))
@@ -455,7 +529,7 @@ class RegexTestDialog(gui.RegexTestDialog):
         event.Skip()
 
     def on_textctrl_selectall(self, event):
-        """Selectall content of textctrl."""
+        """Select all content of `TextCtrl`."""
 
         text = self.FindFocus()
         if isinstance(text, wx.TextCtrl):
@@ -517,7 +591,7 @@ class RegexTestDialog(gui.RegexTestDialog):
             event.Skip()
 
     def on_replace_plugin_dir_changed(self, event):
-        """Handle replace plugin dir change."""
+        """Handle replace plugin directory change."""
 
         pth = event.target
         if pth is not None and os.path.exists(pth):
@@ -529,9 +603,11 @@ class RegexTestDialog(gui.RegexTestDialog):
         if self.m_replace_plugin_checkbox.GetValue():
             self.m_replace_label.SetLabel(self.REPLACE_PLUGIN)
             self.m_replace_plugin_dir_picker.Show()
+            self.m_reload_button.Show()
         else:
             self.m_replace_label.SetLabel(self.REPLACE)
             self.m_replace_plugin_dir_picker.Hide()
+            self.m_reload_button.Hide()
         self.m_tester_panel.GetSizer().Layout()
 
         self.regex_start_event(event)
@@ -546,7 +622,16 @@ class RegexTestDialog(gui.RegexTestDialog):
         else:
             self.regex_start_event(event)
 
-    on_regex_changed = regex_start_event
+    def on_reload_click(self, event):
+        """On script reload click."""
+
+        self.imported_plugin = None
+        self.regex_start_event(event)
+
+    def on_regex_changed(self, event):
+        """On regex changed."""
+
+        self.regex_start_event(event)
 
     on_regex_toggle = regex_start_event
 
