@@ -22,9 +22,8 @@ from __future__ import unicode_literals
 import codecs
 import json
 import os
-import traceback
 import copy
-from . import portalocker
+from filelock import FileLock
 from ..app import custom_app
 from ..app.custom_app import debug, debug_struct, error
 from ..generic_dialogs import errormsg
@@ -33,6 +32,7 @@ from ..localization import _
 from .. import data
 from .. import notify
 from ... import rumcore
+from ...rumcore import text_decode
 from ... import util
 
 DEV_MODE = False
@@ -41,7 +41,7 @@ CACHE_FILE = "rummage_dev.cache" if DEV_MODE else "rummage.cache"
 LOG_FILE = "rummage.log"
 FIFO = "rummage.fifo"
 
-SETTINGS_FMT = '2.1.0'
+SETTINGS_FMT = '2.3.1'
 CACHE_FMT = '2.0.0'
 
 NOTIFY_STYLES = {
@@ -53,6 +53,38 @@ NOTIFY_STYLES = {
 BACKUP_FILE = 0
 BACKUP_FOLDER = 1
 
+DEFAULT_SETTINGS = {
+    "__format__": SETTINGS_FMT,
+    "current_version": (0, 0, 0, 'final', 0, 0),
+    "alert_enabled": True,
+    "backup_ext": "rum-bak",
+    "backup_folder": ".rum-bak",
+    "backup_type": BACKUP_FILE,
+    "brace_expansion": False,
+    "chains": {},
+    "check_prerelease": False,
+    "check_updates": False,
+    "editor": "",
+    "extmatch": False,
+    "file_case_sensitive": False,
+    "full_exclude_path": False,
+    "full_file_path": False,
+    "globstar": False,
+    "hide_cols_content": [],
+    "hide_cols_file": [],
+    "hide_limit": False,
+    "international_time": False,
+    "locale": "en_US",
+    "notify_enabled": True,
+    "notify_method": "default",
+    "regex_mode": rumcore.RE_MODE,
+    "regex_version": 0,
+    "saved_searches": {},
+    "single_instance": False,
+    "term_notifier": "",
+    "encoding_options": copy.deepcopy(text_decode.DEFAULT_ENCODING_OPTIONS)
+}
+
 
 class Settings(object):
     """Handle settings."""
@@ -71,13 +103,11 @@ class Settings(object):
         cls.ERR_SAVE_CACHE_FAILED = _("Failed to save cache file!")
 
     @classmethod
-    def load_settings(cls, debug_mode):
+    def load_settings(cls):
         """Load the settings."""
 
-        cls.debug = debug_mode
         cls.localize()
-        cls.settings_file, cls.cache_file, log = cls.get_settings_files()
-        custom_app.init_app_log(log)
+        cls.setup_setting_files()
         cls.settings = {"__format__": SETTINGS_FMT}
         cls.cache = {"__format__": CACHE_FMT}
         cls.settings_time = None
@@ -86,9 +116,7 @@ class Settings(object):
         if cls.settings_file is not None:
             cls.open_settings()
             cls.open_cache()
-        if cls.debug:
-            custom_app.set_debug_mode(True)
-        localization.setup('rummage', os.path.join(cls.config_folder, "locale"), cls.get_language())
+        localization.setup('rummage', cls.get_language())
         cls.localize()
         debug_struct(cls.settings)
         debug_struct(cls.cache)
@@ -101,12 +129,114 @@ class Settings(object):
         cls.reload_settings()
         return copy.deepcopy(cls.settings)
 
+    @classmethod
+    def get_log_file(cls):
+        """Get location of log file."""
+
+        return cls.log
+
+    @classmethod
+    def set_current_version(cls, value):
+        """Set current version."""
+
+        cls.reload_settings()
+        cls._set_current_version(value)
+        cls.save_settings()
+
+    @classmethod
+    def _set_current_version(cls, value):
+        """Set current version."""
+
+        cls.settings["current_version"] = value
+
+    @classmethod
+    def get_current_version(cls):
+        """Get current version."""
+
+        cls.reload_settings()
+        value = cls.settings.get('current_version', DEFAULT_SETTINGS['current_version'])
+        return value
+
     @classmethod
     def is_regex_available(cls):
         """Check if regex support is available."""
 
         return rumcore.REGEX_SUPPORT
 
+    @classmethod
+    def is_cchardet_available(cls):
+        """Check if `cchardet` is available."""
+
+        return text_decode.CCDetect is not None
+
+    @classmethod
+    def set_chardet_mode(cls, value):
+        """Set `chardet` mode."""
+
+        cls.reload_settings()
+        cls._set_chardet_mode(value)
+        cls.save_settings()
+
+    @classmethod
+    def _set_chardet_mode(cls, value):
+        """Set `chardet` mode."""
+
+        if not cls.is_cchardet_available() or value > text_decode.CHARDET_CLIB:
+            value = text_decode.CHARDET_DEFAULT
+        if 'encoding_options' not in cls.settings:
+            cls.settings['encoding_options'] = copy.deepcopy(text_decode.DEFAULT_ENCODING_OPTIONS)
+        cls.settings["encoding_options"]['chardet_mode'] = value
+
+    @classmethod
+    def get_chardet_mode(cls):
+        """See if regex support is enabled."""
+
+        cls.reload_settings()
+        value = cls.settings.get('encoding_options', {}).get('chardet_mode', text_decode.CHARDET_DEFAULT)
+        if text_decode.CCDetect is None or value > text_decode.CHARDET_CLIB:
+            value = text_decode.CHARDET_DEFAULT
+        return value
+
+    @classmethod
+    def _set_encoding_ext(cls, values):
+        """Set encoding options."""
+
+        if 'encoding_options' not in cls.settings:
+            cls.settings['encoding_options'] = copy.deepcopy(text_decode.DEFAULT_ENCODING_OPTIONS)
+
+        for k, v in values.items():
+            if k != 'chardet_mode' and k in text_decode.DEFAULT_ENCODING_OPTIONS:
+                cls.settings['encoding_options'][k] = v[:]
+
+    @classmethod
+    def set_encoding_ext(cls, values):
+        """Set `chardet` mode."""
+
+        cls.reload_settings()
+        cls._set_encoding_ext(values)
+        cls.save_settings()
+
+    @classmethod
+    def get_encoding_ext(cls):
+        """Get encoding extensions."""
+
+        cls.reload_settings()
+        value = cls.settings.get('encoding_options', {})
+        options = {}
+        for k, v in value.items():
+            if k != "chardet_mode":
+                options[k] = v[:]
+        return options
+
+    @classmethod
+    def get_encoding_options(cls):
+        """Get encoding extensions."""
+
+        cls.reload_settings()
+        options = cls.get_encoding_ext()
+        options['chardet_mode'] = cls.get_chardet_mode()
+        return options
+
     @classmethod
     def set_regex_mode(cls, value):
         """Set regex support."""
@@ -176,6 +306,69 @@ class Settings(object):
 
         cls.settings["hide_limit"] = hide
 
+    @classmethod
+    def get_hide_cols_file(cls):
+        """Get hide cols file."""
+
+        cls.reload_settings()
+        return cls.settings.get("hide_cols_file", [])
+
+    @classmethod
+    def set_hide_cols_file(cls, hide):
+        """Set hide cols file."""
+
+        cls.reload_settings()
+        cls._set_hide_cols_file(sorted(hide))
+        cls.save_settings()
+
+    @classmethod
+    def _set_hide_cols_file(cls, hide):
+        """Set hide cols file."""
+
+        cls.settings["hide_cols_file"] = hide
+
+    @classmethod
+    def get_hide_cols_content(cls):
+        """Get hide cols content."""
+
+        cls.reload_settings()
+        return cls.settings.get("hide_cols_content", [])
+
+    @classmethod
+    def set_hide_cols_content(cls, hide):
+        """Set hide cols content."""
+
+        cls.reload_settings()
+        cls._set_hide_cols_content(sorted(hide))
+        cls.save_settings()
+
+    @classmethod
+    def _set_hide_cols_content(cls, hide):
+        """Set hide cols content."""
+
+        cls.settings["hide_cols_content"] = hide
+
+    @classmethod
+    def get_international_time(cls):
+        """Get international time setting."""
+
+        cls.reload_settings()
+        return cls.settings.get("international_time", False)
+
+    @classmethod
+    def set_international_time(cls, itime):
+        """Set international time setting."""
+
+        cls.reload_settings()
+        cls._set_international_time(itime)
+        cls.save_settings()
+
+    @classmethod
+    def _set_international_time(cls, hide):
+        """Set international time setting."""
+
+        cls.settings["international_time"] = hide
+
     @classmethod
     def get_language(cls):
         """Get locale language."""
@@ -205,7 +398,7 @@ class Settings(object):
         """Return languages."""
 
         languages = []
-        base = os.path.join(cls.config_folder, "locale")
+        base = localization.locale_path
         if os.path.exists(base):
             for file_obj in os.listdir(base):
                 if os.path.isdir(os.path.join(base, file_obj)):
@@ -252,8 +445,8 @@ class Settings(object):
         return changed
 
     @classmethod
-    def get_settings_files(cls):
-        """Get settings, cache, log, and fifo location."""
+    def setup_setting_files(cls):
+        """Get settings, cache, log, and FIFO location."""
 
         platform = util.platform()
 
@@ -264,9 +457,9 @@ class Settings(object):
             plugin_folder = os.path.join(folder, 'plugins')
             if not os.path.exists(plugin_folder):
                 os.mkdir(plugin_folder)
-            settings = os.path.join(folder, SETTINGS_FILE)
-            cache = os.path.join(folder, CACHE_FILE)
-            log = os.path.join(folder, LOG_FILE)
+            cls.settings_file = os.path.join(folder, SETTINGS_FILE)
+            cls.cache_file = os.path.join(folder, CACHE_FILE)
+            cls.log = os.path.join(folder, LOG_FILE)
             cls.fifo = os.path.join(folder, '\\\\.\\pipe\\rummage')
             cls.config_folder = folder
         elif platform == "osx":
@@ -280,9 +473,9 @@ class Settings(object):
             plugin_folder = os.path.join(folder, 'plugins')
             if not os.path.exists(plugin_folder):
                 os.mkdir(plugin_folder)
-            settings = os.path.join(folder, SETTINGS_FILE)
-            cache = os.path.join(folder, CACHE_FILE)
-            log = os.path.join(folder, LOG_FILE)
+            cls.settings_file = os.path.join(folder, SETTINGS_FILE)
+            cls.cache_file = os.path.join(folder, CACHE_FILE)
+            cls.log = os.path.join(folder, LOG_FILE)
             cls.fifo = os.path.join(folder, FIFO)
             cls.config_folder = folder
         elif platform == "linux":
@@ -292,41 +485,32 @@ class Settings(object):
             plugin_folder = os.path.join(folder, 'plugins')
             if not os.path.exists(plugin_folder):
                 os.mkdir(plugin_folder)
-            settings = os.path.join(folder, SETTINGS_FILE)
-            cache = os.path.join(folder, CACHE_FILE)
-            log = os.path.join(folder, LOG_FILE)
+            cls.settings_file = os.path.join(folder, SETTINGS_FILE)
+            cls.cache_file = os.path.join(folder, CACHE_FILE)
+            cls.log = os.path.join(folder, LOG_FILE)
             cls.fifo = os.path.join(folder, FIFO)
             cls.config_folder = folder
 
-        if not os.path.exists(settings):
-            cls.new_settings(settings)
-        if not os.path.exists(cache):
-            cls.new_cache(cache)
+        cls.settings_lock = FileLock(cls.settings_file + '.lock')
+        cls.cache_lock = FileLock(cls.cache_file + '.lock')
 
-        return settings, cache, log
+        if not os.path.exists(cls.settings_file):
+            cls.new_settings(cls.settings_file)
+        if not os.path.exists(cls.cache_file):
+            cls.new_cache(cls.cache_file)
 
     @classmethod
     def open_settings(cls):
         """Open settings file."""
 
         try:
-            locked = False
-            with codecs.open(cls.settings_file, "r", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_SH), "Could not lock settings."
-                locked = True
-                cls.settings = json.loads(f.read())
-                assert portalocker.unlock(f), "Could not unlock settings."
-                locked = False
-                cls.update_settings()
+            with cls.settings_lock.acquire(2):
+                with codecs.open(cls.settings_file, "r", encoding="utf-8") as f:
+                    cls.settings = json.loads(f.read())
         except Exception:
-            e = traceback.format_exc()
-            try:
-                error(e)
-                if locked:
-                    portalocker.unlock(f)
-                errormsg(cls.ERR_LOAD_SETTINGS_FAILED)
-            except Exception:
-                print(str(e))
+            errormsg(cls.ERR_LOAD_SETTINGS_FAILED)
+
+        cls.update_settings()
 
     @classmethod
     def new_settings(cls, settings):
@@ -335,19 +519,12 @@ class Settings(object):
         default_settings = {'__format__': SETTINGS_FMT}
 
         try:
-            locked = False
-            with codecs.open(settings, "w", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_EX), "Could not lock file."
-                locked = True
-                f.write(json.dumps(default_settings, sort_keys=True, indent=4, separators=(',', ': ')))
-                assert portalocker.unlock(f), "Could not unlock file."
-                locked = False
+            with cls.settings_lock.acquire(2):
+                with codecs.open(settings, "w", encoding="utf-8") as f:
+                    f.write(json.dumps(default_settings, sort_keys=True, indent=4, separators=(',', ': ')))
         except Exception:
-            try:
-                if locked:
-                    portalocker.unlock(f)
-            except Exception:
-                pass
+            # It's okay that we couldn't do it this time.
+            pass
 
         return default_settings
 
@@ -355,11 +532,11 @@ class Settings(object):
     def update_settings(cls):
         """Update settings."""
 
-        settings_format = cls.settings.get('__format__')
-        # if settings_format is None:
-        #     # Replace invalid settings
-        #     cls.settings = cls.new_settings(cls.settings_file)
-        if settings_format < '2.1.0':
+        updated = False
+        settings_format = tuple([int(x) for x in cls.settings.get('__format__').split('.')])
+
+        if settings_format < (2, 1, 0):
+            updated = True
             searches = cls.settings.get("saved_searches", {})
 
             # Upgrade versions before format existed
@@ -390,51 +567,53 @@ class Settings(object):
 
             # Update format
             cls.settings["__format__"] = SETTINGS_FMT
+
+        if settings_format < (2, 2, 0):
+            if 'editor' in cls.settings and isinstance(cls.settings['editor'], (list, tuple)):
+                cls.settings['editor'] = ""
+
+        # Update settings with any missing values
+        for k, v in DEFAULT_SETTINGS.items():
+            if k not in cls.settings:
+                updated = True
+                cls.settings[k] = copy.deepcopy(v)
+            elif k == 'encoding_options':
+                for k1, v1 in v.items():
+                    if k1 not in cls.settings[k]:
+                        updated = True
+                        cls.settings[k][k1] = copy.copy(v1)
+
+        if updated:
             cls.save_settings()
 
     @classmethod
     def open_cache(cls):
-        """Open cache file."""
+        """Open settings file."""
 
+        success = False
         try:
-            locked = False
-            with codecs.open(cls.cache_file, "r", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_SH), "Could not lock settings."
-                locked = True
-                cls.cache = json.loads(f.read())
-                assert portalocker.unlock(f), "Could not unlock settings."
-                locked = False
-                cls.update_cache()
+            with cls.cache_lock.acquire(2):
+                with codecs.open(cls.cache_file, "r", encoding="utf-8") as f:
+                    cls.cache = json.loads(f.read())
+            success = True
         except Exception:
-            e = traceback.format_exc()
-            try:
-                error(e)
-                if locked:
-                    portalocker.unlock(f)
-                errormsg(cls.ERR_LOAD_CACHE_FAILED)
-            except Exception:
-                print(str(e))
+            errormsg(cls.ERR_LOAD_CACHE_FAILED)
+        if success:
+            cls.update_cache()
 
     @classmethod
     def new_cache(cls, cache):
-        """New cache."""
+        """New settings."""
 
         default_cache = {'__format__': CACHE_FMT}
 
         try:
-            locked = False
-            with codecs.open(cache, "w", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_EX), "Could not lock file."
-                locked = True
-                f.write(json.dumps(default_cache, sort_keys=True, indent=4, separators=(',', ': ')))
-                assert portalocker.unlock(f), "Could not unlock file."
-                locked = False
+            with cls.cache_lock.acquire(2):
+                with codecs.open(cache, "w", encoding="utf-8") as f:
+                    f.write(json.dumps(default_cache, sort_keys=True, indent=4, separators=(',', ': ')))
         except Exception:
-            try:
-                if locked:
-                    portalocker.unlock(f)
-            except Exception:
-                pass
+            # It's okay that we couldn't do it this time.
+            pass
 
         return default_cache
 
@@ -452,13 +631,13 @@ class Settings(object):
 
     @classmethod
     def get_config_folder(cls):
-        """Return config folder."""
+        """Return configuration folder."""
 
         return cls.config_folder
 
     @classmethod
     def get_fifo(cls):
-        """Get fifo pipe."""
+        """Get FIFO pipe."""
 
         return cls.fifo
 
@@ -471,26 +650,15 @@ class Settings(object):
             settings = None
             cache = None
             if cls.settings_file is not None:
-                locked = False
                 try:
-                    with codecs.open(cls.settings_file, "r", encoding="utf-8") as f:
-                        assert portalocker.lock(f, portalocker.LOCK_SH), "Could not lock settings."
-                        locked = True
-                        settings = json.loads(f.read())
-                        assert portalocker.unlock(f), "could not unlock settings."
-                        locked = False
-                    with codecs.open(cls.cache_file, "r", encoding="utf-8") as f:
-                        assert portalocker.lock(f, portalocker.LOCK_SH), "Could not lock cache file."
-                        locked = True
-                        cache = json.loads(f.read())
-                        assert portalocker.unlock(f), "could not unlock cache file."
-                        locked = False
+                    with cls.settings_lock.acquire(2):
+                        with codecs.open(cls.settings_file, "r", encoding="utf-8") as f:
+                            settings = json.loads(f.read())
+                    with cls.cache_lock:
+                        with codecs.open(cls.cache_file, "r", encoding="utf-8") as f:
+                            cache = json.loads(f.read())
                 except Exception:
-                    try:
-                        if locked:
-                            portalocker.unlock(f)
-                    except Exception:
-                        pass
+                    pass
             if settings is not None:
                 cls.settings = settings
             if cache is not None:
@@ -697,9 +865,9 @@ class Settings(object):
         pth = cls.get_config_folder()
 
         # Clean up old images
-        png = os.path.join(pth, "Rummage-notify.png")
-        icon = os.path.join(pth, "Rummage-notify.ico")
-        icns = os.path.join(pth, "Rummage-notify.icns")
+        png = os.path.join(pth, "rum-notify.png")
+        icon = os.path.join(pth, "rum-notify.ico")
+        icns = os.path.join(pth, "rum-notify.icns")
         for img in (png, icon, icns):
             try:
                 if os.path.exists(img):
@@ -707,30 +875,18 @@ class Settings(object):
             except Exception:
                 pass
 
-        # New file names
-        png = os.path.join(pth, "rum-notify.png")
-        icon = os.path.join(pth, "rum-notify.ico")
-        icns = os.path.join(pth, "rum-notify.icns")
+        png = os.path.join(data.RESOURCE_PATH, "rummage_hires.png")
+        icon = os.path.join(data.RESOURCE_PATH, "rummage_tray.ico")
+        icns = os.path.join(data.RESOURCE_PATH, "rummage.icns")
+        growl_png = os.path.join(data.RESOURCE_PATH, "rummage_large.png")
 
-        try:
-            if not os.path.exists(png):
-                with open(png, "wb") as f:
-                    f.write(data.get_image('rummage_hires.png').GetData())
-        except Exception:
+        if not os.path.exists(png):
             png = None
 
-        try:
-            if not os.path.exists(icon):
-                with open(icon, "wb") as f:
-                    f.write(data.get_image('rummage_tray.ico').GetData())
-        except Exception:
+        if not os.path.exists(icon):
             icon = None
 
-        try:
-            if not os.path.exists(icns):
-                with open(icns, "wb") as f:
-                    f.write(data.get_image('rummage.icns').GetData())
-        except Exception:
+        if not os.path.exists(icns):
             icns = None
 
         # Set up notifications
@@ -741,12 +897,20 @@ class Settings(object):
             os.path.exists(os.path.join(notifier, 'Contents/MacOS/terminal-notifier'))
         ):
             notifier = os.path.join(notifier, 'Contents/MacOS/terminal-notifier')
+
+        if util.platform() == "windows":
+            img = icon
+        elif util.platform() == "osx":
+            img = icns
+        else:
+            img = png
+
         notify.setup_notifications(
             "Rummage",
-            png,
-            icon,
-            (notifier, None, icns)
+            img,
+            (notifier, None)
         )
+        notify.setup_growl_notifications("Rummage", growl_png)
         notify.enable_growl(cls.get_notify_method() == "growl" and notify.has_growl())
 
     @classmethod
@@ -931,6 +1095,70 @@ class Settings(object):
 
         cls.settings['backup_folder'] = value
 
+    @classmethod
+    def get_last_update_check(cls):
+        """Get the last time an update check was performed."""
+
+        cls.reload_settings()
+        return cls.settings.get('last_update_check', None)
+
+    @classmethod
+    def set_last_update_check(cls, value):
+        """Get the last time an update check was performed."""
+
+        cls.reload_settings()
+        cls.settings['last_update_check'] = value
+        cls.save_settings()
+
+    @classmethod
+    def _set_check_updates(cls, value):
+        """Set check updates."""
+
+        cls.settings['check_updates'] = value
+
+    @classmethod
+    def set_check_updates(cls, value):
+        """Set check updates."""
+
+        cls.reload_settings()
+        cls._set_check_updates(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_check_updates(cls):
+        """Get check updates."""
+
+        cls.reload_settings()
+        return cls.settings.get('check_updates', False)
+
+    @classmethod
+    def _set_prerelease(cls, value):
+        """Set check updates."""
+
+        cls.settings['check_prerelease'] = value
+
+    @classmethod
+    def set_prerelease(cls, value):
+        """Set prerelease."""
+
+        cls.reload_settings()
+        cls._set_prerelease(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_prerelease(cls):
+        """Get check updates."""
+
+        cls.reload_settings()
+        return cls.settings.get('check_prerelease', False)
+
+    @classmethod
+    def get_update_options(cls):
+        """Get the update options."""
+
+        cls.reload_settings()
+        return cls.settings.get('check_updates', False), cls.settings.get('check_prerelease', False)
+
     @classmethod
     def get_history_record_count(cls, history_types=None):
         """Get number of history items saved."""
@@ -944,6 +1172,19 @@ class Settings(object):
             count += len(cls.cache.get(h, []))
         return count
 
+    @classmethod
+    def get_history(cls, history_types=None):
+        """Get number of history items saved."""
+
+        if history_types is None:
+            history_types = []
+
+        cls.reload_settings()
+        history = {}
+        for h in history_types:
+            history[h] = cls.cache.get(h, [])
+        return history
+
     @classmethod
     def clear_history_records(cls, history_types=None):
         """Clear history types."""
@@ -957,6 +1198,132 @@ class Settings(object):
                 cls.cache[h] = []
         cls.save_cache()
 
+    @classmethod
+    def _set_extmatch(cls, value):
+        """Set `extmatch`."""
+
+        cls.settings['extmatch'] = value
+
+    @classmethod
+    def set_extmatch(cls, value):
+        """Set `extmatch`."""
+
+        cls.reload_settings()
+        cls._set_extmatch(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_extmatch(cls):
+        """Get `extmatch`."""
+
+        cls.reload_settings()
+        return cls.settings.get('extmatch', False)
+
+    @classmethod
+    def _set_brace_expansion(cls, value):
+        """Set brace_expansion."""
+
+        cls.settings['brace_expansion'] = value
+
+    @classmethod
+    def set_brace_expansion(cls, value):
+        """Set brace_expansion."""
+
+        cls.reload_settings()
+        cls._set_brace_expansion(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_brace_expansion(cls):
+        """Get brace_expansion."""
+
+        cls.reload_settings()
+        return cls.settings.get('brace_expansion', False)
+
+    @classmethod
+    def _set_file_case_sensitive(cls, value):
+        """Set file_case_sensitive."""
+
+        cls.settings['file_case_sensitive'] = value
+
+    @classmethod
+    def set_file_case_sensitive(cls, value):
+        """Set file_case_sensitive."""
+
+        cls.reload_settings()
+        cls._set_file_case_sensitive(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_file_case_sensitive(cls):
+        """Get file_case_sensitive."""
+
+        cls.reload_settings()
+        return cls.settings.get('file_case_sensitive', False)
+
+    @classmethod
+    def _set_full_exclude_path(cls, value):
+        """Set full_exclude_path."""
+
+        cls.settings['full_exclude_path'] = value
+
+    @classmethod
+    def set_full_exclude_path(cls, value):
+        """Set full_exclude_path."""
+
+        cls.reload_settings()
+        cls._set_full_exclude_path(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_full_exclude_path(cls):
+        """Get full_exclude_path."""
+
+        cls.reload_settings()
+        return cls.settings.get('full_exclude_path', False)
+
+    @classmethod
+    def _set_full_file_path(cls, value):
+        """Set full_file_path."""
+
+        cls.settings['full_file_path'] = value
+
+    @classmethod
+    def set_full_file_path(cls, value):
+        """Set full_file_path."""
+
+        cls.reload_settings()
+        cls._set_full_file_path(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_full_file_path(cls):
+        """Get full_file_path."""
+
+        cls.reload_settings()
+        return cls.settings.get('full_file_path', False)
+
+    @classmethod
+    def _set_globstar(cls, value):
+        """Set globstar."""
+
+        cls.settings['globstar'] = value
+
+    @classmethod
+    def set_globstar(cls, value):
+        """Set globstar."""
+
+        cls.reload_settings()
+        cls._set_globstar(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_globstar(cls):
+        """Get globstar."""
+
+        cls.reload_settings()
+        return cls.settings.get('globstar', False)
+
     @classmethod
     def import_settings(cls, obj):
         """Import settings."""
@@ -971,6 +1338,31 @@ class Settings(object):
         if 'backup_type' in obj:
             cls._set_backup_type(obj['backup_type'])
 
+        # Updates
+        if 'check_updates' in obj:
+            cls._set_check_updates(obj['check_updates'])
+        if 'check_prerelease' in obj:
+            cls._set_prerelease(obj['check_prerelease'])
+
+        if 'encoding_options' in obj:
+            if 'chardet_mode' in obj['encoding_options']:
+                cls._set_chardet_mode(obj['encoding_options']['chardet_mode'])
+            cls._set_encoding_ext(obj['encoding_options'])
+
+        # File matching
+        if 'extmatch' in obj:
+            cls._set_extmatch(obj['extmatch'])
+        if 'brace_expansion' in obj:
+            cls._set_brace_expansion(obj['brace_expansion'])
+        if 'file_case_sensitive' in obj:
+            cls._set_file_case_sensitive(obj['file_case_sensitive'])
+        if 'full_exclude_path' in obj:
+            cls._set_full_exclude_path(obj['full_exclude_path'])
+        if 'full_file_path' in obj:
+            cls._set_full_file_path(obj['full_file_path'])
+        if 'globstar' in obj:
+            cls._set_globstar(obj['globstar'])
+
         # Notifications
         update_notify = False
         if 'alert_enabled' in obj:
@@ -1000,6 +1392,16 @@ class Settings(object):
         if 'hide_limit' in obj:
             cls._set_hide_limit(obj['hide_limit'])
 
+        # Hide columns
+        if 'hide_cols_file' in obj:
+            cls._set_hide_cols_file(obj['hide_cols_file'])
+        if 'hide_cols_content' in obj:
+            cls._set_hide_cols_content(obj['hide_cols_content'])
+
+        # International time
+        if 'international_time' in obj:
+            cls._set_international_time(obj['international_time'])
+
         # Regex
         if 'regex_mode' in obj:
             cls._set_regex_mode(obj['regex_mode'])
@@ -1033,49 +1435,27 @@ class Settings(object):
         """Save settings."""
 
         try:
-            locked = False
             cls.settings['__format__'] = SETTINGS_FMT
-            with codecs.open(cls.settings_file, "w", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_EX), "Could not lock settings."
-                locked = True
-                f.write(json.dumps(cls.settings, sort_keys=True, indent=4, separators=(',', ': ')))
-                assert portalocker.unlock(f), "could not unlock settings."
-                locked = False
+            with cls.settings_lock.acquire(2):
+                with codecs.open(cls.settings_file, "w", encoding="utf-8") as f:
+                    f.write(json.dumps(cls.settings, sort_keys=True, indent=4, separators=(',', ': ')))
         except Exception:
-            e = traceback.format_exc()
-            try:
-                if locked:
-                    portalocker.unlock(f)
-                errormsg(cls.ERR_SAVE_SETTINGS_FAILED)
-                error(e)
-            except Exception:
-                print(str(e))
+            errormsg(cls.ERR_SAVE_SETTINGS_FAILED)
 
     @classmethod
     def save_cache(cls):
         """Save cache."""
 
         try:
-            locked = False
             cls.cache['__format__'] = CACHE_FMT
-            with codecs.open(cls.cache_file, "w", encoding="utf-8") as f:
-                assert portalocker.lock(f, portalocker.LOCK_EX), "Could not lock cache file."
-                locked = True
-                f.write(json.dumps(cls.cache, sort_keys=True, indent=4, separators=(',', ': ')))
-                assert portalocker.unlock(f), "could not unlock cache file."
-                locked = False
+            with cls.cache_lock.acquire(2):
+                with codecs.open(cls.cache_file, "w", encoding="utf-8") as f:
+                    f.write(json.dumps(cls.cache, sort_keys=True, indent=4, separators=(',', ': ')))
         except Exception:
-            e = traceback.format_exc()
-            try:
-                if locked:
-                    portalocker.unlock(f)
-                errormsg(cls.ERR_SAVE_CACHE_FAILED)
-                error(e)
-            except Exception:
-                print(str(e))
+            errormsg(cls.ERR_SAVE_CACHE_FAILED)
 
     @classmethod
     def unload(cls):
-        """Perfrom needed actions when done with settings."""
+        """Perform needed actions when done with settings."""
 
         notify.destroy_notifications()
